@@ -1,4 +1,4 @@
-/* $Id: ttkFrame.c,v 1.12.2.2 2010/08/26 02:06:09 hobbs Exp $
+/*
  * Copyright (c) 2004, Joe English
  *
  * ttk::frame and ttk::labelframe widgets.
@@ -44,6 +44,7 @@ static Tk_OptionSpec FrameOptionSpecs[] = {
 	Tk_Offset(Frame,frame.heightObj), -1,
 	0,0,GEOMETRY_CHANGED },
 
+    WIDGET_TAKEFOCUS_FALSE,
     WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
@@ -205,10 +206,9 @@ int TtkGetLabelAnchorFromObj(
 
 error:
     if (interp) {
-	Tcl_ResetResult(interp);
-	Tcl_AppendResult(interp,
-	    "Bad label anchor specification ", Tcl_GetString(objPtr),
-	    NULL);
+	Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+		"Bad label anchor specification %s", Tcl_GetString(objPtr)));
+	Tcl_SetErrorCode(interp, "TTK", "LABEL", "ANCHOR", NULL);
     }
     return TCL_ERROR;
 }
