@@ -25,7 +25,7 @@ int TtkXPTheme_Init(Tcl_Interp *interp, HWND hwnd) { return TCL_OK; }
 
 #include <windows.h>
 #include <uxtheme.h>
-#ifdef HAVE_VSSYM32_H
+#if defined(HAVE_VSSYM32_H) || _MSC_VER > 1500
 #   include <vssym32.h>
 #else
 #   include <tmschema.h>
@@ -106,7 +106,7 @@ LoadXPThemeProcs(HINSTANCE *phlib)
 	 * We have successfully loaded the library. Proceed in storing the
 	 * addresses of the functions we want to use.
 	 */
-	XPThemeProcs *procs = (XPThemeProcs*)ckalloc(sizeof(XPThemeProcs));
+	XPThemeProcs *procs = ckalloc(sizeof(XPThemeProcs));
 #define LOADPROC(name) \
 	(0 != (procs->name = (name ## Proc *)GetProcAddress(handle, #name) ))
 
@@ -124,7 +124,7 @@ LoadXPThemeProcs(HINSTANCE *phlib)
 	    return procs;
 	}
 #undef LOADPROC
-	ckfree((char*)procs);
+	ckfree(procs);
     }
     return 0;
 }
@@ -411,7 +411,7 @@ typedef struct
 static ElementData *
 NewElementData(XPThemeProcs *procs, ElementInfo *info)
 {
-    ElementData *elementData = (ElementData*)ckalloc(sizeof(ElementData));
+    ElementData *elementData = ckalloc(sizeof(ElementData));
 
     elementData->procs = procs;
     elementData->info = info;
@@ -429,10 +429,10 @@ static void DestroyElementData(void *clientData)
 {
     ElementData *elementData = clientData;
     if (elementData->info->flags & HEAP_ELEMENT) {
-	ckfree((char *)elementData->info->statemap);
-	ckfree((char *)elementData->info->className);
-	ckfree((char *)elementData->info->elementName);
-	ckfree((char *)elementData->info);
+	ckfree(elementData->info->statemap);
+	ckfree(elementData->info->className);
+	ckfree(elementData->info->elementName);
+	ckfree(elementData->info);
     }
     ckfree(clientData);
 }
@@ -1062,13 +1062,14 @@ GetSysFlagFromObj(Tcl_Interp *interp, Tcl_Obj *objPtr, int *resultPtr)
     if (Tcl_ListObjGetElements(interp, objPtr, &objc, &objv) != TCL_OK)
 	return TCL_ERROR;
     if (objc != 2) {
-	Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj("wrong # args", -1));
+	Tcl_SetErrorCode(interp, "TCL", "WRONGARGS", NULL);
 	return TCL_ERROR;
     }
     for (i = 0; i < objc; ++i) {
 	int option;
-	if (Tcl_GetIndexFromObj(interp, objv[i], names, "system constant", 0, &option)
-		!= TCL_OK)
+	if (Tcl_GetIndexFromObjStruct(interp, objv[i], names,
+		sizeof(char *), "system constant", 0, &option) != TCL_OK)
 	    return TCL_ERROR;
 	*resultPtr |= (flags[option] << (8 * (1 - i)));
     }
@@ -1109,15 +1110,16 @@ Ttk_CreateVsapiElement(
     LPWSTR wname;
     Ttk_ElementSpec *elementSpec = &GenericElementSpec;
 
-    const char *optionStrings[] =
+    static const char *optionStrings[] =
 	{ "-padding","-width","-height","-margins", "-syssize",
 	  "-halfheight", "-halfwidth", NULL };
     enum { O_PADDING, O_WIDTH, O_HEIGHT, O_MARGINS, O_SYSSIZE,
 	   O_HALFHEIGHT, O_HALFWIDTH };
 
     if (objc < 2) {
-	Tcl_AppendResult(interp,
-	    "missing required arguments 'class' and/or 'partId'", NULL);
+	Tcl_SetObjResult(interp, Tcl_NewStringObj(
+	    "missing required arguments 'class' and/or 'partId'", -1));
+	Tcl_SetErrorCode(interp, "TTK", "VSAPI", "REQUIRED", NULL);
 	return TCL_ERROR;
     }
 
@@ -1132,12 +1134,14 @@ Ttk_CreateVsapiElement(
 	for (i = 3; i < objc; i += 2) {
 	    int tmp = 0;
 	    if (i == objc -1) {
-		Tcl_AppendResult(interp, "Missing value for \"",
-			Tcl_GetString(objv[i]), "\".", NULL);
+		Tcl_SetObjResult(interp, Tcl_ObjPrintf(
+			"Missing value for \"%s\".",
+			Tcl_GetString(objv[i])));
+		Tcl_SetErrorCode(interp, "TTK", "VSAPI", "MISSING", NULL);
 		return TCL_ERROR;
 	    }
-	    if (Tcl_GetIndexFromObj(interp, objv[i], optionStrings,
-		    "option", 0, &option) != TCL_OK)
+	    if (Tcl_GetIndexFromObjStruct(interp, objv[i], optionStrings,
+		    sizeof(char *), "option", 0, &option) != TCL_OK)
 		return TCL_ERROR;
 	    switch (option) {
 	    case O_PADDING:
@@ -1197,8 +1201,7 @@ Ttk_CreateVsapiElement(
 	if (Tcl_ListObjGetElements(interp, objv[2], &count, &specs) != TCL_OK)
 	    return TCL_ERROR;
 	/* we over-allocate to ensure there is a terminating entry */
-	stateTable = (Ttk_StateTable *)
-		ckalloc(sizeof(Ttk_StateTable) * (count + 1));
+	stateTable = ckalloc(sizeof(Ttk_StateTable) * (count + 1));
 	memset(stateTable, 0, sizeof(Ttk_StateTable) * (count + 1));
 	for (n = 0, j = 0; status == TCL_OK && n < count; n += 2, ++j) {
 	    Ttk_StateSpec spec = {0,0};
@@ -1211,15 +1214,15 @@ Ttk_CreateVsapiElement(
 	    }
 	}
 	if (status != TCL_OK) {
-	    ckfree((char *)stateTable);
+	    ckfree(stateTable);
 	    return status;
 	}
     } else {
-	stateTable = (Ttk_StateTable *)ckalloc(sizeof(Ttk_StateTable));
+	stateTable = ckalloc(sizeof(Ttk_StateTable));
 	memset(stateTable, 0, sizeof(Ttk_StateTable));
     }
 
-    elementPtr = (ElementInfo *)ckalloc(sizeof(ElementInfo));
+    elementPtr = ckalloc(sizeof(ElementInfo));
     elementPtr->elementSpec = elementSpec;
     elementPtr->partId = partId;
     elementPtr->statemap = stateTable;
@@ -1232,7 +1235,7 @@ Ttk_CreateVsapiElement(
     elementPtr->elementName = name;
 
     /* set the class name to an allocated copy */
-    wname = (LPWSTR) ckalloc(sizeof(WCHAR) * (length + 1));
+    wname = ckalloc(sizeof(WCHAR) * (length + 1));
     wcscpy(wname, className);
     elementPtr->className = wname;
 
@@ -1279,7 +1282,7 @@ MODULE_SCOPE int TtkXPTheme_Init(Tcl_Interp *interp, HWND hwnd)
      * Set theme data and cleanup proc
      */
 
-    themeData = (XPThemeData *)ckalloc(sizeof(XPThemeData));
+    themeData = ckalloc(sizeof(XPThemeData));
     themeData->procs = procs;
     themeData->hlibrary = hlibrary;
 
